/*

  WINGDB.CPP - Garmin MapSource GDB file parser for Windows - works with MapSource v 6.3

  Performs various route and track conversions. Based on ParseGDB v 2.5.

  Some of the information on the GDB format was provided by Ian Cowley and AnnoyedOne

  This program is derived from my observations of the data files produced by MapSource, and 
  is used at your own risk. No warranty is offered, expressed or implied.

  Copyright 2004 Simon Long, KuDaTa Software
*/

#include "stdafx.h"
#include "wingdb.h"
#include "parsegdb.h"

/*---------------------------------------------------------------------------*/
/* CWinGDBApp class - application */
/*---------------------------------------------------------------------------*/

/* Static definition of one instance */

CWinGDBApp theApp;

/* Constructor - does nothing */

CWinGDBApp::CWinGDBApp ()
{
}

/* InitInstance - opens a CWinGDBDlg dialog window */

BOOL CWinGDBApp::InitInstance ()
{
	CWinGDBDlg dlg;
	m_pMainWnd = &dlg;
	dlg.DoModal ();
	return FALSE;
}


/*---------------------------------------------------------------------------*/
/* CWinGDBDlg class - main window */
/*---------------------------------------------------------------------------*/

BEGIN_MESSAGE_MAP(CWinGDBDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_BN_CLICKED(IDC_BUTTON1, OnInfileButton)
	ON_BN_CLICKED(IDC_BUTTON2, OnOutfileButton)
	ON_BN_CLICKED(IDC_BUTTON3, OnConvertButton)
END_MESSAGE_MAP()

/* Constructor - loads the icon */

CWinGDBDlg::CWinGDBDlg (CWnd* pParent) : CDialog (CWinGDBDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp ()->LoadIcon (IDR_ALTICON);
}

/* OnInitDialog - sets up dialog initial state */

BOOL CWinGDBDlg::OnInitDialog ()
{
	CDialog::OnInitDialog ();

	/* Add the About item to the system menu */
	CMenu* pSysMenu = GetSystemMenu (FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString (IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty ())
		{
			pSysMenu->AppendMenu (MF_SEPARATOR);
			pSysMenu->AppendMenu (MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
	
	/* Set the dialog icon */
	SetIcon (m_hIcon, TRUE);
	SetIcon (m_hIcon, FALSE);
	
	/* Set the default mode on the radio buttons */
	CButton *defRad = reinterpret_cast <CButton *> (GetDlgItem (IDC_RADIO1));
	defRad->SetCheck (TRUE);
	
	return TRUE;
}

/* OnSysCommand - handler for About... menu command */

void CWinGDBDlg::OnSysCommand (UINT nID, LPARAM lParam)
{
	/* Handle calls to open the About box */
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal ();
	}
	else CDialog::OnSysCommand (nID, lParam);
}

/* OnInfileButton - handler for input file browse button */

void CWinGDBDlg::OnInfileButton () 
{
	/* Open the input file browser */
	CFileDialog dlgFile (TRUE, "*.gdb", NULL, OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, 
		"GDB Files (*.gdb)|*.gdb|All Files (*.*)|*.*||", this);

	if (dlgFile.DoModal () == IDOK)
	{
		/* Copy the input file name to the text box */
		CEdit *infile = reinterpret_cast <CEdit *> (GetDlgItem (IDC_EDIT1));
		infile->SetWindowText (dlgFile.GetPathName ());
	}
}

/* OnOutfileButton - handler for output file browse button */

void CWinGDBDlg::OnOutfileButton () 
{
	/* Open the output file browser */
	CFileDialog dlgFile (FALSE, "*.gdb", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
		"GDB Files (*.gdb)|*.gdb|All Files (*.*)|*.*||", this);

	if (dlgFile.DoModal() == IDOK)
	{
		/* Copy the output file name to the text box */
		CEdit *outfile = reinterpret_cast <CEdit *> (GetDlgItem (IDC_EDIT2));
		outfile->SetWindowText (dlgFile.GetPathName ());
	}	
}

/* OnConvertButton - handler for file convert button */

void CWinGDBDlg::OnConvertButton () 
{
	FILE *finp, *foutp;
	char filename[1024];
	char mode;
	LPSTR pFilename = filename;
	
	/* Read the input filename */
	CEdit *infile = reinterpret_cast <CEdit *> (GetDlgItem (IDC_EDIT1));
	infile->GetWindowText (pFilename, 1023);
	if (*filename == 0) 
	{
		MessageBox ("No input file specified", "File Error", MB_ICONEXCLAMATION);
		return;
	}

	/* Open the input file */
	finp = fopen (filename, "rb");
	if (finp == NULL) 
	{
		MessageBox ("Can't open input file", "File Error", MB_ICONEXCLAMATION);
		return;
	}

	/* Read the output filename */
	CEdit *outfile = reinterpret_cast <CEdit *> (GetDlgItem (IDC_EDIT2));
	outfile->GetWindowText (pFilename, 1023);
	if (*filename == 0) 
	{
		MessageBox ("No output file specified", "File Error", MB_ICONEXCLAMATION);
		return;
	}

	/* Open the output file */
	foutp = fopen (filename, "wb");
	if (foutp == NULL) 
	{
		MessageBox ("Can't open output file", "File Error", MB_ICONEXCLAMATION);
		fclose (finp);
		return;
	}

	/* Get the radio button state */
	switch (GetCheckedRadioButton (IDC_RADIO1, IDC_RADIO6))
	{
		case IDC_RADIO1:	mode = 'T';
							break;

		case IDC_RADIO2:	mode = 'U';
							break;

		case IDC_RADIO3:	mode = 'R';
							break;

		case IDC_RADIO4:	mode = 'S';
							break;

		case IDC_RADIO5:	mode = 'V';
							break;

		case IDC_RADIO6:	mode = 'W';
							break;

		default:			mode = 'T';
							break;
	}

	/* Copy the file header */
	if (copy_header (finp, foutp))
	{
		/* Scan chunks until chunk type 'V' is found - always seems to be last in file */
		while (process_chunk (finp, foutp, mode) != 'V');
	}
	else 
	{
		/* Warn of bad file format */
		MessageBox ("Input file is not in GDB format", "File Error", MB_ICONEXCLAMATION);
		fclose (finp);
		fclose (foutp);
		return;
	}

	/* Close files */
	fclose (foutp);
	fclose (finp);

	MessageBox ("GDB file converted successfully", "Finished", MB_OK);
}


/*---------------------------------------------------------------------------*/
/* CAboutDlg class - about box */
/*---------------------------------------------------------------------------*/

/* Constructor - does nothing */

CAboutDlg::CAboutDlg () : CDialog (CAboutDlg::IDD)
{
}


/* End of file */